@@ -95,7 +95,7 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
   PROTECT(cmatch = match(tnodes, cnodes, 0));
   c = INTEGER(cmatch);
   /* sorting takes care of different node orderings. */
-  R_isort(c, nnodes);
+  R_qsort_int(c, 1, nnodes);
 
   /* check that every node in the first network is also present in the
    * second one; this is enough because the node sets have the same size
@@ -141,8 +141,8 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
     t = INTEGER(thash);
     c = INTEGER(chash);
     /* sorting takes care of different arc orderings. */
-    R_isort(t, narcs);
-    R_isort(c, narcs);
+    R_qsort_int(t, 1, narcs);
+    R_qsort_int(c, 1, narcs);
 
     /* compare the integer vectors as generic memory areas. */
     if (memcmp(t, c, narcs * sizeof(int))) {
