@@ -1,15 +1,17 @@
 #include "include/rcore.h"
 #include "include/globals.h"
-#include "include/dataframe.h"
+#include "include/data.frame.h"
 #include "include/sampling.h"
 #include "include/matrix.h"
 #include "include/fitted.h"
+#include "include/data.table.h"
 
 /* predict the value of a gaussian node without parents. */
 SEXP gpred(SEXP fitted, SEXP ndata, SEXP debug) {
 
-int i = 0, *n = INTEGER(ndata), debuglevel = isTRUE(debug);
+int i = 0, *n = INTEGER(ndata);
 double *mean = NULL, *res = NULL;
+bool debugging = isTRUE(debug);
 SEXP result;
 
   /* get the (only) coefficient of the linear regression. */
@@ -23,12 +25,9 @@ SEXP result;
   for (i = 0; i < *n; i++)
     res[i] = *mean;
 
-  if (debuglevel > 0) {
-
+  if (debugging)
     Rprintf("  > prediction for all observations is %lf.\n", *mean);
 
-  }/*THEN*/
-
   UNPROTECT(1);
 
   return result;
@@ -38,50 +37,45 @@ SEXP result;
 /* predict the value of a gaussian node with one or more parents. */
 SEXP cgpred(SEXP fitted, SEXP parents, SEXP debug)  {
 
-int i = 0, j = 0, ndata = length(VECTOR_ELT(parents, 0)), ncol = length(parents);
-int debuglevel = isTRUE(debug);
+int i = 0, j = 0;
 double *res = NULL, *coefs = NULL;
-double **columns = NULL;
+bool debugging = isTRUE(debug);
 SEXP result;
+gdata dt = { 0 };
 
   /* get the coefficient of the linear regression. */
   coefs = REAL(getListElement(fitted, "coefficients"));
-
+  /* extract the columns from the data frame. */
+  dt = gdata_from_SEXP(parents, 0);
   /* allocate and initialize the return value. */
-  PROTECT(result = allocVector(REALSXP, ndata));
+  PROTECT(result = allocVector(REALSXP, dt.m.nobs));
   res = REAL(result);
 
-  /* dereference the columns of the data frame. */
-  columns = (double **) Calloc1D(ncol, sizeof(double));
-  for (i = 0; i < ncol; i++)
-    columns[i] = REAL(VECTOR_ELT(parents, i));
-
-  for (i = 0; i < ndata; i++) {
+  for (i = 0; i < dt.m.nobs; i++) {
 
     /* compute the mean value for this observation. */
     res[i] = coefs[0];
 
-    for (j = 0; j < ncol; j++)
-      res[i] += columns[j][i] * coefs[j + 1];
+    for (j = 0; j < dt.m.ncols; j++)
+      res[i] += dt.col[j][i] * coefs[j + 1];
 
-    if (debuglevel > 0) {
+    if (debugging) {
 
       Rprintf("  > prediction for observation %d is %lf with predictor:\n",
         i + 1, res[i]);
 
-      Rprintf("    (%lf) + (%lf) * (%lf)", coefs[0], columns[0][i], coefs[1]);
-      for (j = 1; j < ncol; j++)
-        Rprintf(" + (%lf) * (%lf)", columns[j][i], coefs[j + 1]);
+      Rprintf("    (%lf) + (%lf) * (%lf)", coefs[0], dt.col[0][i], coefs[1]);
+      for (j = 1; j < dt.m.ncols; j++)
+        Rprintf(" + (%lf) * (%lf)", dt.col[j][i], coefs[j + 1]);
       Rprintf("\n");
 
     }/*THEN*/
 
   }/*FOR*/
 
+  FreeGDT(dt, FALSE);
   UNPROTECT(1);
 
-  Free1D(columns);
-
   return result;
 
 }/*CGPRED*/
@@ -90,9 +84,9 @@ SEXP result;
 SEXP dpred(SEXP fitted, SEXP ndata, SEXP prob, SEXP debug) {
 
 int i = 0, nmax = 0, n = INT(ndata), length = 0;
-int *res = NULL, debuglevel = isTRUE(debug), include_prob = isTRUE(prob);
-int *iscratch = NULL, *maxima = NULL, tr_nlevels = 0;
+int *iscratch = NULL, *maxima = NULL, tr_nlevels = 0, *res = NULL;
 double *cpt = NULL, *dscratch = NULL, *buf = NULL, *pt = NULL;
+bool debugging = isTRUE(debug), include_prob = isTRUE(prob);
 SEXP ptab, result, tr_levels, probtab = R_NilValue;
 
   /* get the probabilities of the multinomial distribution. */
@@ -124,7 +118,7 @@ SEXP ptab, result, tr_levels, probtab = R_NilValue;
   tr_nlevels = length(tr_levels);
 
   /* allocate and initialize the table of the prediction probabilities. */
-  if (include_prob > 0) {
+  if (include_prob) {
 
     PROTECT(probtab = allocMatrix(REALSXP, tr_nlevels, n));
     pt = REAL(probtab);
@@ -139,7 +133,7 @@ SEXP ptab, result, tr_levels, probtab = R_NilValue;
     for (i = 0; i < n; i++)
       res[i] = maxima[0];
 
-    if (debuglevel > 0) {
+    if (debugging) {
 
       if (res[0] == NA_INTEGER)
         Rprintf("  > prediction for all observations is NA with probabilities:\n");
@@ -162,7 +156,7 @@ SEXP ptab, result, tr_levels, probtab = R_NilValue;
     SampleReplace(n, nmax, res, maxima);
     PutRNGstate();
 
-    if (debuglevel > 0) {
+    if (debugging) {
 
       Rprintf("  > there are %d levels tied for prediction, applying tie breaking.\n", nmax);
       Rprintf("  > tied levels are:");
@@ -174,7 +168,7 @@ SEXP ptab, result, tr_levels, probtab = R_NilValue;
 
   }/*ELSE*/
 
-  if (include_prob > 0) {
+  if (include_prob) {
 
     /* set the levels of the target variable as rownames. */
     setDimNames(probtab, tr_levels, R_NilValue);
@@ -202,11 +196,11 @@ SEXP ptab, result, tr_levels, probtab = R_NilValue;
 /* predict the value of a discrete node with one or more parents. */
 SEXP cdpred(SEXP fitted, SEXP parents, SEXP prob, SEXP debug) {
 
-int i = 0, k = 0, n = length(parents), nrow = 0, ncol = 0;
-int *configs = INTEGER(parents), debuglevel = isTRUE(debug);
-int tr_nlevels = 0, include_prob = isTRUE(prob);
+int i = 0, k = 0, n = length(parents), nrow = 0, ncol = 0, tr_nlevels = 0;
+int *configs = INTEGER(parents);
 int *iscratch = NULL, *maxima = NULL, *nmax = NULL, *res = NULL;
 double *cpt = NULL, *dscratch = NULL, *buf = NULL, *pt = NULL;
+bool debugging = isTRUE(debug), include_prob = isTRUE(prob);
 SEXP temp, result, tr_levels, probtab = R_NilValue;
 
   /* get the probabilities of the multinomial distribution. */
@@ -250,7 +244,7 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
   tr_nlevels = length(tr_levels);
 
   /* allocate and initialize the table of the prediction probabilities. */
-  if (include_prob > 0) {
+  if (include_prob) {
 
     PROTECT(probtab = allocMatrix(REALSXP, tr_nlevels, n));
     pt = REAL(probtab);
@@ -267,7 +261,7 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
 
       res[i] = NA_INTEGER;
 
-      if (debuglevel > 0)
+      if (debugging)
         Rprintf("  > prediction for observation %d is NA because the probabilities are missing.\n");
 
     }/*THEN*/
@@ -275,7 +269,7 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
 
       res[i] = maxima[CMC(0, configs[i] - 1, nrow)];
 
-      if (debuglevel > 0) {
+      if (debugging) {
 
         if (res[i] == NA_INTEGER)
           Rprintf("  > prediction for observation %d is NA with probabilities:\n");
@@ -296,7 +290,7 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
       /* break ties: sample with replacement from all the maxima. */
       SampleReplace(1, nmax[configs[i] - 1], res + i, maxima + (configs[i] - 1) * nrow);
 
-      if (debuglevel > 0) {
+      if (debugging) {
 
         Rprintf("  > there are %d levels tied for prediction of observation %d, applying tie breaking.\n",
           nmax[configs[i] - 1], i + 1);
@@ -322,7 +316,7 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
   /* save the state of the random number generator. */
   PutRNGstate();
 
-  if (include_prob > 0) {
+  if (include_prob) {
 
     /* set the levels of the target variable as rownames. */
     setDimNames(probtab, tr_levels, R_NilValue);
@@ -351,44 +345,47 @@ SEXP temp, result, tr_levels, probtab = R_NilValue;
 /* predict the values of a conditional Gaussian node. */
 SEXP ccgpred(SEXP fitted, SEXP configurations, SEXP parents, SEXP debug) {
 
-int i = 0, j = 0, ndata = length(configurations);
-int *config = INTEGER(configurations), debuglevel = isTRUE(debug);
-int cur_config = 0, np = length(parents), nrow = np + 1;
-double *res = NULL, *beta = NULL, *beta_offset = NULL, **columns = NULL;
+int i = 0, j = 0, cur_config = 0;
+int *config = INTEGER(configurations);
+double *res = NULL, *beta = NULL, *beta_offset = NULL;
+bool debugging = isTRUE(debug);
 SEXP result;
+gdata dt = { 0 };
 
   /* get the regression coefficients of the conditional Gaussian distribution. */
   beta = REAL(getListElement(fitted, "coefficients"));
+  /* extract the columns of the data frame. */
+  dt = gdata_from_SEXP(parents, 0);
 
-  /* dereference the columns of the data frame and get the sample size. */
-  columns = (double **) Calloc1D(np, sizeof(double *));
-  for (i = 0; i < np; i++)
-    columns[i] = REAL(VECTOR_ELT(parents, i));
+  /* if there are no continuous parents the data table is empty; reset the
+   * sample size using the discrete parents configurations for consistency. */
+  if ((dt.m.nobs == 0) && (dt.m.ncols == 0))
+    dt.m.nobs = length(configurations);
 
-  /* allocate and initialize the return value. */
-  PROTECT(result = allocVector(REALSXP, ndata));
+  /* allocate the return value. */
+  PROTECT(result = allocVector(REALSXP, dt.m.nobs));
   res = REAL(result);
 
-  for (i = 0; i < ndata; i++)  {
+  for (i = 0; i < dt.m.nobs; i++)  {
 
     /* find out which conditional regression to use for prediction. */
     cur_config = config[i] - 1;
-    beta_offset = beta + cur_config * nrow;
+    beta_offset = beta + cur_config * (dt.m.ncols + 1);
 
     /* compute the mean value for this observation. */
     res[i] = beta_offset[0];
 
-    for (j = 0; j < np; j++)
-      res[i] += columns[j][i] * beta_offset[j + 1];
+    for (j = 0; j < dt.m.ncols; j++)
+      res[i] += dt.col[j][i] * beta_offset[j + 1];
 
-    if (debuglevel > 0) {
+    if (debugging) {
 
       Rprintf("  > prediction for observation %d is %lf with predictor:\n",
         i + 1, res[i]);
 
       Rprintf("    (%lf)", beta_offset[0]);
-      for (j = 0; j < np; j++)
-        Rprintf(" + (%lf) * (%lf)", columns[j][i], beta_offset[j + 1]);
+      for (j = 0; j < dt.m.ncols; j++)
+        Rprintf(" + (%lf) * (%lf)", dt.col[j][i], beta_offset[j + 1]);
       Rprintf("\n");
 
     }/*THEN*/
@@ -397,7 +394,7 @@ SEXP result;
 
   UNPROTECT(1);
 
-  Free1D(columns);
+  FreeGDT(dt, FALSE);
 
   return result;
 
@@ -410,14 +407,15 @@ SEXP naivepred(SEXP fitted, SEXP data, SEXP parents, SEXP training, SEXP prior,
 
 int i = 0, j = 0, k = 0, n = 0, nvars = length(fitted), nmax = 0, tr_nlevels = 0;
 int *res = NULL, **ex = NULL, *ex_nlevels = NULL;
-int idx = 0, *tr_id = INTEGER(training), include_prob = isTRUE(prob);
-int *iscratch = NULL, *maxima = NULL, *prn = NULL, debuglevel = isTRUE(debug);
+int idx = 0, *tr_id = INTEGER(training);
+int *iscratch = NULL, *maxima = NULL, *prn = NULL;
+bool debugging = isTRUE(debug), include_prob = isTRUE(prob);
 double **cpt = NULL, *pr = NULL, *scratch = NULL, *buf = NULL, *pt = NULL;
 double sum = 0;
 SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
 
   /* cache the node labels. */
-  nodes = getAttrib(fitted, R_NamesSymbol);
+  PROTECT(nodes = getAttrib(fitted, R_NamesSymbol));
 
   /* cache the pointers to all the variables. */
   ex = (int **) Calloc1D(nvars, sizeof(int *));
@@ -443,7 +441,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
   /* get the prior distribution. */
   pr = REAL(prior);
 
-  if (debuglevel > 0) {
+  if (debugging) {
 
     Rprintf("* the prior distribution for the target variable is:\n");
     PrintValue(prior);
@@ -474,7 +472,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
   res = INTEGER(result);
 
   /* allocate and initialize the table of the posterior probabilities. */
-  if (include_prob > 0) {
+  if (include_prob) {
 
     PROTECT(probtab = allocMatrix(REALSXP, tr_nlevels, n));
     pt = REAL(probtab);
@@ -496,7 +494,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
 
     }/*FOR*/
 
-    if (debuglevel > 0)
+    if (debugging)
       Rprintf("* predicting the value of observation %d.\n", i + 1);
 
     /* ... and for each conditional probability table... */
@@ -512,7 +510,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
         /* ... and for each row of the conditional probability table... */
         for (k = 0; k < tr_nlevels; k++) {
 
-          if (debuglevel > 0) {
+          if (debugging) {
 
             Rprintf("  > node %s: picking cell %d (%d, %d) from the CPT (p = %lf).\n",
               NODE(j), CMC(ex[j][i] - 1, k, ex_nlevels[j]), ex[j][i], k + 1,
@@ -537,7 +535,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
           idx = (ex[j][i] - 1) + k * ex_nlevels[j] +
                   (ex[prn[j] - 1][i] - 1) * ex_nlevels[j] * tr_nlevels;
 
-          if (debuglevel > 0) {
+          if (debugging) {
 
             Rprintf("  > node %s: picking cell %d (%d, %d, %d) from the CPT (p = %lf).\n",
               NODE(j), idx, ex[j][i], k + 1, ex[prn[j] - 1][i], cpt[j][idx]);
@@ -577,7 +575,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
 
       res[i] = NA_INTEGER;
 
-      if (debuglevel > 0)
+      if (debugging)
         Rprintf("  > prediction for observation %d is NA because the probabilities are missing.\n");
 
     }/*THEN*/
@@ -585,7 +583,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
 
       res[i] = maxima[0];
 
-      if (debuglevel > 0) {
+      if (debugging) {
 
         Rprintf("  @ prediction for observation %d is '%s' with (log-)posterior:\n",
           i + 1, CHAR(STRING_ELT(tr_levels, res[i] - 1)));
@@ -603,7 +601,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
       /* break ties: sample with replacement from all the maxima. */
       SampleReplace(1, nmax, res + i, maxima);
 
-      if (debuglevel > 0) {
+      if (debugging) {
 
         Rprintf("  @ there are %d levels tied for prediction of observation %d, applying tie breaking.\n", nmax, i + 1);
 
@@ -629,7 +627,7 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
   /* add back the attributes and the class to the return value. */
   setAttrib(result, R_LevelsSymbol, tr_levels);
 
-  switch(r_fitted_node_label(tr_node)) {
+  switch(fitted_node_to_enum(tr_node)) {
 
     case DNODE:
       setAttrib(result, R_ClassSymbol, mkString("factor"));
@@ -652,12 +650,12 @@ SEXP temp, tr, tr_levels, tr_node, result, nodes, probtab = R_NilValue;
     /* add the posterior probabilities to the return value. */
     setAttrib(result, BN_ProbSymbol, probtab);
 
-    UNPROTECT(2);
+    UNPROTECT(3);
 
   }/*THEN*/
   else {
 
-    UNPROTECT(1);
+    UNPROTECT(2);
 
   }/*ELSE*/
 
